# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-18 08:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='City',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
        migrations.CreateModel(
            name='CityGeoNameEN',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('city', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='en', to='codenerix_geodata.City')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CityGeoNameES',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('city', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='es', to='codenerix_geodata.City')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Continent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('code', models.CharField(max_length=2, unique=True, verbose_name='Code')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
        migrations.CreateModel(
            name='ContinentGeoNameEN',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('continent', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='en', to='codenerix_geodata.Continent')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContinentGeoNameES',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('continent', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='es', to='codenerix_geodata.Continent')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('code', models.CharField(max_length=2, unique=True, verbose_name='Code')),
                ('continent', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='countries', to='codenerix_geodata.Continent', verbose_name='Continent')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
        migrations.CreateModel(
            name='CountryGeoNameEN',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('country', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='en', to='codenerix_geodata.Country')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CountryGeoNameES',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('country', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='es', to='codenerix_geodata.Country')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Province',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('code', models.CharField(max_length=3, verbose_name='Code')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
        migrations.CreateModel(
            name='ProvinceGeoNameEN',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('province', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='en', to='codenerix_geodata.Province')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProvinceGeoNameES',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('province', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='es', to='codenerix_geodata.Province')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Region',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('code', models.CharField(max_length=3, verbose_name='Code')),
                ('country', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='regions', to='codenerix_geodata.Country', verbose_name='Country')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
        migrations.CreateModel(
            name='RegionGeoNameEN',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('region', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='en', to='codenerix_geodata.Region')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RegionGeoNameES',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('region', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='es', to='codenerix_geodata.Region')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TimeZone',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=50, unique=True, verbose_name='Name')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
        migrations.AddField(
            model_name='province',
            name='region',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='provinces', to='codenerix_geodata.Region', verbose_name='Region'),
        ),
        migrations.AddField(
            model_name='city',
            name='country',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cities', to='codenerix_geodata.Country', verbose_name='Country'),
        ),
        migrations.AddField(
            model_name='city',
            name='province',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='cities', to='codenerix_geodata.Province', verbose_name='Province'),
        ),
        migrations.AddField(
            model_name='city',
            name='region',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='cities', to='codenerix_geodata.Region', verbose_name='Region'),
        ),
        migrations.AddField(
            model_name='city',
            name='time_zone',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cities', to='codenerix_geodata.TimeZone', verbose_name='City'),
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix-geodata
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" Populate geographic data using the free database provided by MaxMind.

Two different databases were used:
    - GeoLite2 City: http://geolite.maxmind.com/download/geoip/database/GeoLite2-City-CSV.zip
    - GeoLite2 Country: http://geolite.maxmind.com/download/geoip/database/GeoLite2-Country-CSV.zip
"""

import sys
import time
from os.path import dirname, join
from csv import reader

from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils.translation import ugettext as _
from django.core.exceptions import ObjectDoesNotExist

from codenerix_lib.debugger import Debugger
from codenerix_extensions.helpers import FileBZ2
from codenerix_geodata.models import Continent, Country, Region, Province, City, TimeZone


BASE_LANGUAGE = 'EN'

COUNTRY_DATA_FILES = {
    'ES': 'GeoLite2-Country-Locations-es.csv.bz2',
    'EN': 'GeoLite2-Country-Locations-en.csv.bz2',
}

CITY_DATA_FILES = {
    'ES': 'GeoLite2-City-Locations-es.csv.bz2',
    'EN': 'GeoLite2-City-Locations-en.csv.bz2',
}

LANGUAGES = set([code for code in settings.LANGUAGES_DATABASES if code in COUNTRY_DATA_FILES])


def clean(name):
    if name:
        try:
            name = name.decode('utf-8')
        except Exception:
            pass
        if name[0] == '"' and name[-1] == '"':
            return name[1:-1]
    return name


def populate_missing_names(data):
    for lang in LANGUAGES:
        for d in data.values():
            if lang not in d or d[lang] == '':
                for other_lang in LANGUAGES - {lang}:
                    if d[other_lang] != '':
                        d[lang] = d[other_lang]
                        break


def continents_lines(filename):
    with FileBZ2(filename, 'rb') as data_file:
        csv_file = reader(data_file, delimiter=',', quotechar='"')

        first = True
        for line in csv_file:
            if first:
                first = False
                continue

            _, _, continent_code, continent_name, _, _ = line
            if continent_code.strip() != '' and continent_name.strip() != '':
                yield continent_code, clean(continent_name)


def country_lines(filename):
    with FileBZ2(filename, 'rb') as data_file:
        csv_file = reader(data_file, delimiter=',', quotechar='"')

        first = True
        for line in csv_file:
            if first:
                first = False
                continue

            geoid, _, continent_code, _, country_code, country_name = line
            if continent_code.strip() != '' and country_code.strip() != '' and country_name.strip() != '':
                yield int(geoid), continent_code, country_code, clean(country_name)


def region_lines(filename):
    with FileBZ2(filename, 'rb') as data_file:
        csv_file = reader(data_file, delimiter=',', quotechar='"')

        first = True
        for line in csv_file:
            if first:
                first = False
                continue

            geoid, _, _, _, country_code, _, region_code, region_name, province_code, province_name, city_name, _, _ = line
            if country_code != '' and region_code != '' and region_name.strip() != '':
                yield int(geoid), country_code, region_code, clean(region_name)


def province_lines(filename):
    with FileBZ2(filename, 'rb') as data_file:
        csv_file = reader(data_file, delimiter=',', quotechar='"')

        first = True
        for line in csv_file:
            if first:
                first = False
                continue

            geoid, _, _, _, country_code, _, region_code, _, province_code, province_name, _, _, _ = line
            if country_code != '' and region_code != '' and province_code != '' and province_name != '':
                yield int(geoid), country_code, region_code, province_code, clean(province_name)


def city_lines(filename):
    with FileBZ2(filename, 'rb') as data_file:
        csv_file = reader(data_file, delimiter=',', quotechar='"')

        first = True
        for line in csv_file:
            if first:
                first = False
                continue

            geoid, _, _, _, country_code, _, region_code, _, province_code, _, city_name, _, time_zone = line
            if geoid != '' and city_name.strip() != '' and time_zone != '':
                yield int(geoid), country_code, region_code, province_code, clean(city_name), clean(time_zone)


class Command(BaseCommand, Debugger):
    help = _('Populates Continent, Country and City models')
    __percent = None

    def percent_init(self, text, total):
        self.__percent = {}
        self.__percent['text'] = text
        self.__percent['last'] = time.time()-6
        self.__percent['counter'] = 0
        self.__percent['total'] = total
        self.percent()

    def percent(self):
        last = (self.__percent['counter']+1) == self.__percent['total']
        now = time.time()
        if (self.__percent['last']+6<now) or last:
            self.__percent['last'] = now
            if self.__percent['counter']:
                delstr=""
                for i in range(0, 200):
                    delstr+="\b"
                sys.stdout.write(delstr)
            if not last:
                text = "{}/{}".format(self.__percent['counter'], self.__percent['total'])
                percent = 100*self.__percent['counter']/self.__percent['total']
                self.debug("{}: ".format(self.__percent['text']), color="blue", tail=False)
                self.debug("{}".format(text), color="cyan", head=False, tail=False)
                self.debug(" - {:.1f}%      ".format(percent), color="white", head=False, tail=False)
            else:
                self.debug(self.__percent['text'], color="blue", tail=False)
                self.debug(" ... Done                     ", head=False, color="green")
            sys.stdout.flush()
        self.__percent['counter'] += 1

    def handle(self, *args, **options):

        # Autoconfigure Debugger
        self.set_name('CODENERIX-GEODATA')
        self.set_debug()

        # print('Erasing existing data ...')
        # City.objects.all().delete()
        # TimeZone.objects.all().delete()
        # Province.objects.all().delete()
        # Region.objects.all().delete()
        # Country.objects.all().delete()
        # Continent.objects.all().delete()

        print('Importing new data ... This action may take some minutes.')
        print('')
        data_path = join(dirname(dirname(dirname(__file__))), 'data')

        # Importing language generated models
        for lang in LANGUAGES:
            exec('from codenerix_geodata.models import ContinentGeoName{}'.format(lang))
            exec('from codenerix_geodata.models import CountryGeoName{}'.format(lang))
            exec('from codenerix_geodata.models import RegionGeoName{}'.format(lang))
            exec('from codenerix_geodata.models import ProvinceGeoName{}'.format(lang))
            exec('from codenerix_geodata.models import CityGeoName{}'.format(lang))

        self.debug('Importing ...', color='yellow', tail=False)
        self.debug(' Continents', color='purple', head=False, tail=False)
        self.debug(' Countries Regions Provinces Cities', color='grey', head=False)
        continents = {}
        for lang in LANGUAGES:
            filename = join(data_path, COUNTRY_DATA_FILES[lang])
            lines = list(continents_lines(filename))
            self.percent_init("    > Prepare data {}".format(lang), len(lines))
            for code, name in lines:
                if code not in continents:
                    continents[code] = {
                        'model': Continent(code=code)
                    }
                continents[code][lang] = name
                self.percent()

        items = continents.items()
        self.percent_init("    > Link", len(items))
        for code, continent in items:
            try:
                model = Continent.objects.get(code=code)
                continent['model'] = model
            except ObjectDoesNotExist:
                continent['model'].save()
            self.percent()

        for lang in LANGUAGES:
            model_type = eval('ContinentGeoName{}'.format(lang))
            items = continents.values()
            self.percent_init("    > Fill {}".format(lang), len(items))
            for continent in items:
                try:
                    model = model_type.objects.get(continent=continent['model'])
                except ObjectDoesNotExist:
                    model = model_type()
                    model.continent = continent['model']
                model.name = continent[lang]
                model.save()
                self.percent()

        self.debug('Importing ...', color='yellow', tail=False)
        self.debug(' Continents', color='simplepurple', head=False, tail=False)
        self.debug(' Countries', color='purple', head=False, tail=False)
        self.debug(' Regions Provinces Cities', color='grey', head=False)
        countries = {}
        for lang in LANGUAGES:
            filename = join(data_path, COUNTRY_DATA_FILES[lang])
            items = list(country_lines(filename))
            self.percent_init("    > Prepare data {}".format(lang), len(items))
            for geoid, continent, code, name in items:
                if code not in countries:
                    countries[code] = {
                        'model': Country(
                            pk=geoid,
                            code=code,
                            continent=continents[continent]['model']
                        )
                    }
                countries[code][lang] = name
                self.percent()

        values = countries.values()
        self.percent_init("    > Link", len(values))
        for country in values:
            try:
                model = Country.objects.get(code=country['model'].code)
                country['model'] = model
            except ObjectDoesNotExist:
                country['model'].save()
            self.percent()

        for lang in LANGUAGES:
            model_type = eval('CountryGeoName{}'.format(lang))
            self.percent_init("    > Fill {}".format(lang), len(values))
            for country in values:
                try:
                    model = model_type.objects.get(country=country['model'])
                except ObjectDoesNotExist:
                    model = model_type()
                    model.country = country['model']
                model.name = country[lang]
                model.save()
                self.percent()

        self.debug('Importing ...', color='yellow', tail=False)
        self.debug(' Continents Countries', color='simplepurple', head=False, tail=False)
        self.debug(' Regions', color='purple', head=False, tail=False)
        self.debug(' Provinces Cities', color='grey', head=False)
        regions = {}
        for lang in LANGUAGES:
            filename = join(data_path, CITY_DATA_FILES[lang])
            lines = list(region_lines(filename))
            self.percent_init("    > Prepare data {}".format(lang), len(lines))
            for geoid, country_code, region_code, region_name in lines:
                region_key = '{}_{}'.format(country_code, region_code)
                if region_key not in regions:
                    regions[region_key] = {
                        'model': Region(
                            pk=geoid,
                            code=region_code,
                            country=countries[country_code]['model']
                        )
                    }
                regions[region_key][lang] = region_name
                self.percent()

        items = regions.values()
        self.percent_init("    > Link", len(items))
        for region in items:
            try:
                model = Region.objects.get(pk=region['model'].pk)
                region['model'] = model
            except ObjectDoesNotExist:
                region['model'].save()
            self.percent()

        self.debug("    > Populate missing", color='blue', tail=False)
        populate_missing_names(regions)
        self.debug(" ... Done", color='green', head=False)

        for lang in LANGUAGES:
            model_type = eval('RegionGeoName{}'.format(lang))
            items = regions.values()
            self.percent_init("    > Fill {}".format(lang), len(items))
            for region in items:
                try:
                    model = model_type.objects.get(region=region['model'])
                except ObjectDoesNotExist:
                    model = model_type()
                    model.region = region['model']
                    model.name = region[lang]
                model.save()
                self.percent()

        self.debug('Importing ...', color='yellow', tail=False)
        self.debug(' Continents Countries Regions', color='simplepurple', head=False, tail=False)
        self.debug(' Provinces', color='purple', head=False, tail=False)
        self.debug(' Cities', color='grey', head=False)
        provinces = {}
        for lang in LANGUAGES:
            filename = join(data_path, CITY_DATA_FILES[lang])
            lines = list(province_lines(filename))
            self.percent_init("    > Prepare data {}".format(lang), len(lines))
            for geoid, country_code, region_code, province_code, province_name in lines:
                region_key = '{}_{}'.format(country_code, region_code)
                province_key = '{}_{}_{}'.format(country_code, region_code, province_code)
                if province_key not in provinces:
                    provinces[province_key] = {
                        'model': Province(
                            pk=geoid,
                            code=province_code,
                            region=regions[region_key]['model']
                        )
                    }
                provinces[province_key][lang] = province_name
                self.percent()

        items = provinces.values()
        self.percent_init("    > Link", len(items))
        for province in items:
            try:
                model = Province.objects.get(pk=province['model'].pk)
                province['model'] = model
            except ObjectDoesNotExist:
                province['model'].save()
            self.percent()

        self.debug("    > Populate missing", color='blue', tail=False)
        populate_missing_names(provinces)
        self.debug(" ... Done", color='green', head=False)

        for lang in LANGUAGES:
            model_type = eval('ProvinceGeoName{}'.format(lang))
            items = provinces.values()
            self.percent_init("    > Fill {}".format(lang), len(items))
            for province in items:
                try:
                    model = model_type.objects.get(province=province['model'])
                except ObjectDoesNotExist:
                    model = model_type()
                    model.province = province['model']
                model.name = province[lang]
                model.save()
                self.percent()

        self.debug('Importing ...', color='yellow', tail=False)
        self.debug(' Continents Countries Regions Provinces', color='simplepurple', head=False, tail=False)
        self.debug(' Cities', color='purple', head=False, tail=False)
        cities = {}
        timezones = {}
        for lang in LANGUAGES:
            filename = join(data_path, CITY_DATA_FILES[lang])
            lines = list(city_lines(filename))
            self.percent_init("    > Prepare data {}".format(lang), len(lines))
            for city_id, country_code, region_code, province_code, city_name, time_zone in lines:
                if time_zone not in timezones:
                    try:
                        model = TimeZone.objects.get(name=time_zone)
                        timezones[time_zone] = model
                    except ObjectDoesNotExist:
                        timezones[time_zone] = TimeZone(name=time_zone)
                        timezones[time_zone].save()

                if city_id not in cities:
                    city = City(
                        pk=city_id,
                        country=countries[country_code]['model'],
                        time_zone=timezones[time_zone]
                    )
                    if region_code != '':
                        region_key = '{}_{}'.format(country_code, region_code)
                        city.region = regions[region_key]['model']

                    if province_code != '':
                        province_key = '{}_{}_{}'.format(country_code, region_code, province_code)
                        city.province = provinces[province_key]['model']

                    cities[city_id] = {
                        'model': city
                    }

                cities[city_id][lang] = city_name
                self.percent()

        items = cities.values()
        self.percent_init("    > Link", len(items))
        for city in items:
            try:
                model = City.objects.get(pk=city['model'].pk)
                city['model'] = model
            except ObjectDoesNotExist:
                city['model'].save()
            self.percent()

        self.debug("    > Populate missing", color='blue', tail=False)
        populate_missing_names(cities)
        self.debug(" ... Done", color='green', head=False)

        for lang in LANGUAGES:
            model_type = eval('CityGeoName{}'.format(lang))
            items = cities.values()
            self.percent_init("    > Fill {}".format(lang), len(items))
            for city in items:
                try:
                    model = model_type.objects.get(city=city['model'])
                except ObjectDoesNotExist:
                    model = model_type()
                    model.city = city['model']
                model.name = city[lang]
                model.save()
                self.percent()

        self.debug('Removing regions without cities ...', color='yellow', tail=False)
        for region in Region.objects.all():
            if region.cities.count() == 0:
                region.delete()
        self.debug(" ... Done", color='green', head=False)

        self.debug('Removing provinces without cities ...', color='yellow', tail=False)
        for province in Province.objects.all():
            if province.cities.count() == 0:
                province.delete()
        self.debug(" ... Done", color='green', head=False)

        self.debug('All done !!!', color="green")



# -*- coding: utf-8 -*-
#
# django-codenerix-geodata
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ast
import operator
from functools import reduce

from django.db.models import Q
from django.conf import settings

from django.utils.translation import ugettext as _

from codenerix.multiforms import MultiForm
from codenerix.views import GenList, GenCreate, GenCreateModal, GenUpdate, GenUpdateModal, GenDelete, GenForeignKey

from .models import Continent, Country, Region, Province, TimeZone, City, MODELS
from .forms import ContinentForm, CountryForm, RegionForm, ProvinceForm, TimeZoneForm, CityForm


# forms for multiforms
formsfull = {}

for info in MODELS:
    field = info[0]
    model = info[1]
    formsfull[model] = [(None, None, None)]
    for lang_code in settings.LANGUAGES_DATABASES:
        query = 'from .models import {}GeoName{}\n'.format(model, lang_code)
        query += 'from .forms import {}TextForm{}'.format(model, lang_code)
        exec(query)

        formsfull[model].append((eval('{}TextForm{}'.format(model, lang_code.upper())), field, None))


class TranslatedMixin(object):

    @property
    def lang(self):
        if hasattr(self, 'request'):
            for x in settings.LANGUAGES:
                if x[0] == self.request.LANGUAGE_CODE:
                    return self.request.LANGUAGE_CODE.lower()
        return settings.LANGUAGES[0][0].lower()


# ###########################################
# Continent
class GenContinentUrl(object):
    ws_entry_point = '{}/continents'.format(settings.CDNX_GEODATA_URL)


class ContinentList(TranslatedMixin, GenContinentUrl, GenList):
    model = Continent
    linkadd = False
    show_details = False
    public = True

    def __fields__(self, info):
        fields = self.model().__fields__(info)
        fields.append(
            ('{}__name'.format(self.lang), _('Name')),
        )
        return fields

    def __searchQ__(self, info, text):
        filters = self.model().__searchQ__(info, text)
        filters['name'] = Q(**{"{}__name__icontains".format(self.lang): text})
        return filters

    def __searchF__(self, info):
        def f(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        filters = self.model().__searchF__(info)
        filters['{}__name'.format(self.lang)] = (_('Name'), f, 'input')
        return filters

    def dispatch(self, request, *args, **kwargs):
        self.order_by = ['{}__name'.format(self.lang)]
        return super(ContinentList, self).dispatch(request, *args, **kwargs)


class ContinentCreate(GenContinentUrl, MultiForm, GenCreate):
    model = Continent
    form_class = ContinentForm
    forms = formsfull['Continent']


class ContinentCreateModal(GenCreateModal, ContinentCreate):
    pass


class ContinentUpdate(GenContinentUrl, MultiForm, GenUpdate):
    model = Continent
    form_class = ContinentForm
    forms = formsfull['Continent']


class ContinentUpdateModal(GenUpdateModal, ContinentUpdate):
    pass


class ContinentDelete(GenContinentUrl, GenDelete):
    model = Continent


# ###########################################
# Country
class GenCountryUrl(object):
    ws_entry_point = '{}/countries'.format(settings.CDNX_GEODATA_URL)


class CountryList(TranslatedMixin, GenCountryUrl, GenList):
    model = Country
    linkadd = False
    show_details = False
    public = True

    def __limitQ__(self, info):
        result = {}
        try:
            params = ast.literal_eval(info.request.GET.get('json'))
        except ValueError:
            params = {}

        continent = int(params.get('continent', 0))

        if continent:
            result['continent_limit'] = Q(continent__pk=continent)
        return result

    def __fields__(self, info):
        return [
            ('code', _('Code')),
            ('{}__name'.format(self.lang), _('Name')),
            ('continent__code', _('Code continent')),
            ('continent__{}__name'.format(self.lang), _('Name continent')),
        ]

    def __searchQ__(self, info, text):
        filters = self.model().__searchQ__(info, text)
        filters['name'] = Q(**{"{}__name__icontains".format(self.lang): text})
        filters['continent_code'] = Q(**{"continent__code": text})
        filters['continent_name'] = Q(**{"continent__{}__name__icontains".format(self.lang): text})
        return filters

    def __searchF__(self, info):
        def f(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        def fc(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"continent__{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        filters = self.model().__searchF__(info)
        filters['{}__name'.format(self.lang)] = (_('Name'), f, 'input')
        filters['continent__code'] = (_('Code continent'), lambda x: Q(continent__code__icontains=x), 'input')
        filters['continent__{}__name'.format(self.lang)] = (_('Continent name'), fc, 'input')
        return filters

    def dispatch(self, request, *args, **kwargs):
        self.order_by = ['{}__name'.format(self.lang)]
        return super(CountryList, self).dispatch(request, *args, **kwargs)


class CountryCreate(GenCountryUrl, MultiForm, GenCreate):
    model = Country
    form_class = CountryForm
    forms = formsfull['Country']


class CountryCreateModal(GenCreateModal, CountryCreate):
    pass


class CountryUpdate(GenCountryUrl, MultiForm, GenUpdate):
    model = Country
    form_class = CountryForm
    forms = formsfull['Country']


class CountryUpdateModal(GenUpdateModal, CountryUpdate):
    pass


class CountryDelete(GenCountryUrl, GenDelete):
    model = Country


class CountryForeign(TranslatedMixin, GenCountryUrl, GenForeignKey):
    model = Country
    label = "{<LANGUAGE_CODE>__name}"
    public = True

    def get_foreign(self, queryset, search, filters):
        # Filter with search string
        qsobject = Q(code__istartswith=search)

        for lang in settings.LANGUAGES_DATABASES:
            qsobject |= Q(**{"{}__name__istartswith".format(lang.lower()): search})
        qs = queryset.filter(qsobject).order_by("{}__name".format(self.lang))
        return qs[:settings.LIMIT_FOREIGNKEY]


# ###########################################
# Regions
class GenRegionUrl(object):
    ws_entry_point = '{}/regions'.format(settings.CDNX_GEODATA_URL)


class RegionList(TranslatedMixin, GenRegionUrl, GenList):
    model = Region
    linkadd = False
    show_details = False
    public = True

    def __limitQ__(self, info):
        result = {}
        try:
            params = ast.literal_eval(info.request.GET.get('json'))
        except ValueError:
            params = {}

        continent = int(params.get('continent', 0))
        country = int(params.get('country', 0))

        if continent:
            result['continent_limit'] = Q(country__continent__pk=continent)
        if country:
            result['country_limit'] = Q(country__pk=country)
        return result

    def __fields__(self, info):
        return [
            ('code', _('Code')),
            ('{}__name'.format(self.lang), _('Name')),
            ('country__code', _('Code country')),
            ('country__{}__name'.format(self.lang), _('Name country')),
            ('country__continent__code', _('Code continent')),
            ('country__continent__{}__name'.format(self.lang), _('Name continent')),
        ]

    def __searchQ__(self, info, text):
        filters = self.model().__searchQ__(info, text)
        filters['name'] = Q(**{"{}__name__icontains".format(self.lang): text})
        filters['country_code'] = Q(**{'country__code': text})
        filters['country_name'] = Q(**{'country__{}__name__icontains'.format(self.lang): text})
        filters['continent_code'] = Q(**{"country__continent__code": text})
        filters['continent_name'] = Q(**{"country__continent__{}__name__icontains".format(self.lang): text})
        return filters

    def __searchF__(self, info):
        def f(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        def fco(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"country__continent__{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        def fcu(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"country__{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        filters = self.model().__searchF__(info)
        filters['{}__name'.format(self.lang)] = (_('Name'), f, 'input')

        filters['country__code'] = (_('Code self'), lambda x: Q(country__code__icontains=x), 'input')
        filters['country__{}__name'.format(self.lang)] = (_('Name country'), fcu, 'input')

        filters['country__continent__code'] = (_('Code continent'), lambda x: Q(country__continent__code__icontains=x), 'input')
        filters['country__continent__{}__name'.format(self.lang)] = (_('Name continent'), fco, 'input')
        return filters

    def dispatch(self, request, *args, **kwargs):
        self.order_by = ['{}__name'.format(self.lang)]
        return super(RegionList, self).dispatch(request, *args, **kwargs)


class RegionCreate(GenRegionUrl, MultiForm, GenCreate):
    model = Region
    form_class = RegionForm
    forms = formsfull['Region']


class RegionCreateModal(GenCreateModal, RegionCreate):
    pass


class RegionUpdate(GenRegionUrl, MultiForm, GenUpdate):
    model = Region
    form_class = RegionForm
    forms = formsfull['Region']


class RegionUpdateModal(GenUpdateModal, RegionUpdate):
    pass


class RegionDelete(GenRegionUrl, GenDelete):
    model = Region


class RegionForeign(TranslatedMixin, GenRegionUrl, GenForeignKey):
    model = Region
    label = "{<LANGUAGE_CODE>__name}"
    public = True

    def get_foreign(self, queryset, search, filters):
        # Filter with search string
        qsobject = Q(code__istartswith=search)

        for lang in settings.LANGUAGES_DATABASES:
            qsobject |= Q(**{"{}__name__istartswith".format(lang.lower()): search})
        qs = queryset.filter(qsobject)

        country = filters.get('country', None)
        if country:
            qs = qs.filter(country__pk=country)
        qs = qs.order_by("{}__name".format(self.lang))
        return qs[:settings.LIMIT_FOREIGNKEY]


# ###########################################
# Provinces
class GenProvinceUrl(object):
    ws_entry_point = '{}/provinces'.format(settings.CDNX_GEODATA_URL)


class ProvinceList(TranslatedMixin, GenProvinceUrl, GenList):
    model = Province
    linkadd = False
    show_details = False
    public = True

    def __limitQ__(self, info):
        result = {}
        try:
            params = ast.literal_eval(info.request.GET.get('json'))
        except ValueError:
            params = {}

        continent = int(params.get('continent', 0))
        country = int(params.get('country', 0))
        region = int(params.get('region', 0))

        if continent:
            result['continent_limit'] = Q(region__country__continent__pk=continent)
        if country:
            result['country_limit'] = Q(region__country__pk=country)
        if region:
            result['region_limit'] = Q(region__pk=region)
        return result

    def __fields__(self, info):
        return [
            ('code', _('Code')),
            ('{}__name'.format(self.lang), _('Name')),
            ('region__code', _('Code region')),
            ('region__{}__name'.format(self.lang), _('Name region')),
            ('region__country__code', _('Code country')),
            ('region__country__{}__name'.format(self.lang), _('Name country')),
            ('region__country__continent__code', _('Code continent')),
            ('region__country__continent__{}__name'.format(self.lang), _('Name continent')),
        ]

    def __searchQ__(self, info, text):
        filters = self.model().__searchQ__(info, text)
        filters['name'] = Q(**{"{}__name__icontains".format(self.lang): text})
        filters['region_code'] = Q(**{'region__code': text})
        filters['region_name'] = Q(**{'region__{}__name__icontains'.format(self.lang): text})
        filters['country_code'] = Q(**{'region__country__code': text})
        filters['country_name'] = Q(**{'region__country__{}__name__icontains'.format(self.lang): text})
        filters['continent_code'] = Q(**{"region__country__continent__code": text})
        filters['continent_name'] = Q(**{"region__country__continent__{}__name__icontains".format(self.lang): text})
        return filters

    def __searchF__(self, info):
        def f(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        def fr(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"region__{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        def fco(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"region__country__continent__{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        def fcu(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"region__country__{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        filters = self.model().__searchF__(info)
        filters['{}__name'.format(self.lang)] = (_('Name'), f, 'input')

        filters['region__code'] = (_('Code self'), lambda x: Q(region__code__icontains=x), 'input')
        filters['region__{}__name'.format(self.lang)] = (_('Name region'), fr, 'input')

        filters['region__country__code'] = (_('Code self'), lambda x: Q(region__country__code__icontains=x), 'input')
        filters['region__country__{}__name'.format(self.lang)] = (_('Name country'), fcu, 'input')

        filters['region__country__continent__code'] = (_('Code continent'), lambda x: Q(region__country__continent__code__icontains=x), 'input')
        filters['region__country__continent__{}__name'.format(self.lang)] = (_('Name continent'), fco, 'input')
        return filters

    def dispatch(self, request, *args, **kwargs):
        self.order_by = ['{}__name'.format(self.lang)]
        return super(ProvinceList, self).dispatch(request, *args, **kwargs)


class ProvinceCreate(GenProvinceUrl, MultiForm, GenCreate):
    model = Province
    form_class = ProvinceForm
    forms = formsfull['Province']


class ProvinceCreateModal(GenCreateModal, ProvinceCreate):
    pass


class ProvinceUpdate(GenProvinceUrl, MultiForm, GenUpdate):
    model = Province
    form_class = ProvinceForm
    forms = formsfull['Province']


class ProvinceUpdateModal(GenUpdateModal, ProvinceUpdate):
    pass


class ProvinceDelete(GenProvinceUrl, GenDelete):
    model = Province


class ProvinceForeign(TranslatedMixin, GenProvinceUrl, GenForeignKey):
    model = Province
    label = "{<LANGUAGE_CODE>__name}"
    public = True

    def get_foreign(self, queryset, search, filters):
        # Filter with search string
        qsobject = Q(code__istartswith=search)

        for lang in settings.LANGUAGES_DATABASES:
            qsobject |= Q(**{"{}__name__istartswith".format(lang.lower()): search})
        qs = queryset.filter(qsobject)

        region = filters.get('region', None)
        if region:
            qs = qs.filter(region__pk=region)
        qs = qs.order_by("{}__name".format(self.lang))
        return qs[:settings.LIMIT_FOREIGNKEY]


# ###########################################
# TimeZone
class GenTimeZoneUrl(object):
    ws_entry_point = '{}/timezones'.format(settings.CDNX_GEODATA_URL)


class TimeZoneList(GenTimeZoneUrl, GenList):
    model = TimeZone
    linkadd = False
    show_details = False
    public = True

    def dispatch(self, request, *args, **kwargs):
        self.order_by = ['name']
        return super(TimeZoneList, self).dispatch(request, *args, **kwargs)


class TimeZoneCreate(GenTimeZoneUrl, GenCreate):
    model = TimeZone
    form_class = TimeZoneForm


class TimeZoneCreateModal(GenCreateModal, TimeZoneCreate):
    pass


class TimeZoneUpdate(GenTimeZoneUrl, GenUpdate):
    model = TimeZone
    form_class = TimeZoneForm


class TimeZoneUpdateModal(GenUpdateModal, TimeZoneUpdate):
    pass


class TimeZoneDelete(GenTimeZoneUrl, GenDelete):
    model = TimeZone


# ###########################################
# City
class GenCityUrl(object):
    ws_entry_point = '{}/cities'.format(settings.CDNX_GEODATA_URL)


class CityList(TranslatedMixin, GenCityUrl, GenList):
    model = City
    linkadd = False
    show_details = False
    public = True

    def __limitQ__(self, info):
        result = {}
        try:
            params = ast.literal_eval(info.request.GET.get('json'))
        except ValueError:
            params = {}

        continent = int(params.get('continent', 0))
        country = int(params.get('country', 0))
        region = int(params.get('region', 0))
        province = int(params.get('province', 0))
        time_zone = int(params.get('time_zone', 0))

        if continent:
            result['continent_limit'] = Q(region__country__continent__pk=continent)
        if country:
            result['country_limit'] = Q(country__pk=country)
        if region:
            result['region_limit'] = Q(region__pk=region)
        if province:
            result['province_limit'] = Q(province__pk=province)
        if time_zone:
            result['time_zone_limit'] = Q(time_zone__pk=time_zone)

        # result['country'] = Q(**{"country__{}__name__icontains".format(self.lang): u'españa'})
        # result['country'] = Q(**{"country__es__name__icontains".format(self.lang): u'españa'})
        return result

    def __fields__(self, info):
        return [
            ('{}__name'.format(self.lang), _('Name')),
            ('time_zone', _('Time Zone')),
            ('province__code'.format(self.lang), _('Code province')),
            ('province__{}__name'.format(self.lang), _('Province')),
            ('country__code'.format(self.lang), _('Code country')),
            ('country__{}__name'.format(self.lang), _('Country')),
            ('country__continent__code', _('Code continent')),
            ('country__continent__{}__name'.format(self.lang), _('Name continent')),
        ]

    def __searchQ__(self, info, text):
        filters = self.model().__searchQ__(info, text)
        filters['name'] = Q(**{"{}__name__icontains".format(self.lang): text})
        filters['province_code'] = Q(**{'province__code': text})
        filters['province_name'] = Q(**{'province__{}__name__icontains'.format(self.lang): text})
        filters['country_code'] = Q(**{'country__code': text})
        filters['country_name'] = Q(**{'country__{}__name__icontains'.format(self.lang): text})
        filters['continent_code'] = Q(**{"country__continent__code": text})
        filters['continent_name'] = Q(**{"country__continent__{}__name__icontains".format(self.lang): text})
        return filters

    def __searchF__(self, info):
        def f(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        def fp(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"province__{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        def fco(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"country__continent__{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        def fcu(x):
            qsobject = []
            for lang in settings.LANGUAGES_DATABASES:
                qsobject.append(Q(**{"country__{}__name__icontains".format(lang.lower()): x}))
            return reduce(operator.or_, qsobject)

        filters = self.model().__searchF__(info)
        filters['{}__name'.format(self.lang)] = (_('Name'), f, 'input')

        filters['province__code'] = (_('Code self'), lambda x: Q(province__code__icontains=x), 'input')
        filters['province__{}__name'.format(self.lang)] = (_('Name region'), fp, 'input')

        filters['country__code'] = (_('Code self'), lambda x: Q(country__code__icontains=x), 'input')
        filters['country__{}__name'.format(self.lang)] = (_('Name country'), fcu, 'input')

        filters['country__continent__code'] = (_('Code continent'), lambda x: Q(country__continent__code__icontains=x), 'input')
        filters['country__continent__{}__name'.format(self.lang)] = (_('Name continent'), fco, 'input')
        return filters

    def dispatch(self, request, *args, **kwargs):
        self.order_by = ['{}__name'.format(self.lang)]
        return super(CityList, self).dispatch(request, *args, **kwargs)


class CityCreate(GenCityUrl, MultiForm, GenCreate):
    model = City
    form_class = CityForm
    forms = formsfull['City']


class CityCreateModal(GenCreateModal, CityCreate):
    pass


class CityUpdate(GenCityUrl, MultiForm, GenUpdate):
    model = City
    form_class = CityForm
    forms = formsfull['City']


class CityUpdateModal(GenUpdateModal, CityUpdate):
    pass


class CityDelete(GenCityUrl, GenDelete):
    model = City


class CityForeign(TranslatedMixin, GenCityUrl, GenForeignKey):
    model = City
    label = "{<LANGUAGE_CODE>__name}"
    public = True
    
    def get_foreign(self, queryset, search, filters):
        # Filter with search string
        qsobject = []
        for lang in settings.LANGUAGES_DATABASES:
            qsobject.append(Q(**{"{}__name__istartswith".format(lang.lower()): search}))
        qs = queryset.filter(
            reduce(operator.or_, qsobject)
        )
        # qs = queryset.filter(qsobject)

        country = filters.get('country', None)
        if country:
            qs = qs.filter(country__pk=country)

        province = filters.get('province', None)
        if province:
            qs = qs.filter(province__pk=province)

        region = filters.get('region', None)
        if region:
            qs = qs.filter(region__pk=region)
        qs = qs.order_by("{}__name".format(self.lang))
        return qs[:settings.LIMIT_FOREIGNKEY]



# -*- coding: utf-8 -*-
#
# django-codenerix-geodata
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from codenerix.forms import GenModelForm

from .models import Continent, Country, Region, Province, TimeZone, City, MODELS


class ContinentForm(GenModelForm):
    class Meta:
        model = Continent
        exclude = []

    def __groups__(self):
        g = [
            (
                _('Details'), 12,
                ['code', 6],
            )
        ]
        return g


class CountryForm(GenModelForm):
    class Meta:
        model = Country
        exclude = []

    def __groups__(self):
        return [
            (
                _('Details'), 12,
                ['continent', 6],
                ['code', 6],
            )
        ]


class RegionForm(GenModelForm):
    class Meta:
        model = Region
        exclude = []

    def __groups__(self):
        return [
            (
                _('Details'), 12,
                ['country', 6],
                ['code', 6],
            )
        ]


class ProvinceForm(GenModelForm):
    class Meta:
        model = Province
        exclude = []

    def __groups__(self):
        return [
            (
                _('Details'), 12,
                ['region', 6],
                ['code', 6],
            )
        ]


class TimeZoneForm(GenModelForm):
    class Meta:
        model = TimeZone
        exclude = []

    def __groups__(self):
        return [
            (
                _('Details'), 12,
                ['name', 6],
            )
        ]


class CityForm(GenModelForm):
    class Meta:
        model = City
        exclude = []

    def __groups__(self):
        return [
            (
                _('Details'), 12,
                ['country', 6],
                ['region', 6],
                ['province', 6],
                ['time_zone', 6],
            )
        ]


# MODELS
query = ""
forms_dyn = []
for info in MODELS:
    field = info[0]
    model = info[1]
    for lang_code in settings.LANGUAGES_DATABASES:
        query = "from .models import {}GeoName{}\n".format(model, lang_code)
        exec(query)
        query = """
class {model}TextForm{lang}(GenModelForm):\n
    class Meta:\n
        model={model}GeoName{lang}\n
        exclude = []\n
    def __groups__(self):\n
        return [(_('Details'),12,"""
        if lang_code == settings.LANGUAGES_DATABASES[0]:
            query += """
                ['name', 12, None, None, None, None, None, ["ng-blur=refresh_lang_field('name', '{model}TextForm', [{languages}])"]],
            )]\n"""
        else:
            query += """
                ['name', 12],
            )]\n"""

        exec(query.format(model=model, lang=lang_code, languages="'{}'".format("','".join(settings.LANGUAGES_DATABASES))))



# -*- coding: utf-8 -*-
#
# django-codenerix-geodata
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from django.db import models
from django.db.models import Q
from django.utils.encoding import smart_text
from django.utils.translation import ugettext_lazy as _

from codenerix.models import CodenerixModel, GenInterface
from codenerix_extensions.helpers import get_language_database


class GenGeoName(CodenerixModel):  # META: Abstract class

    class Meta(CodenerixModel.Meta):
        abstract = True

    name = models.CharField(_('Name'), max_length=100, blank=False)

    def __str__(self):
        return u'{}'.format(smart_text(self.name))

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        return [
            ('name', _('Name'), 100),
        ]


class Continent(CodenerixModel):
    code = models.CharField(_('Code'), max_length=2, unique=True, blank=False)

    def __str__(self):
        lang = get_language_database()
        lang_obj = getattr(self, '{}'.format(lang), None)
        if lang_obj and lang_obj.name:
            txt = lang_obj.name
        else:
            txt = self.code
        return u"{}".format(smart_text(txt))

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        return [
            ('code', _('Code'), 100),
        ]

    def __searchQ__(self, info, text):
        return {
            'code': models.Q(code__icontains=text),
        }

    def __searchF__(self, info):
        filters = {}
        filters['code'] = (_('Code'), lambda x: Q(code__icontains=x), 'input')
        return filters


class Country(CodenerixModel):
    code = models.CharField(_('Code'), max_length=2, unique=True, blank=False)
    continent = models.ForeignKey(Continent, on_delete=models.CASCADE, verbose_name=_('Continent'), related_name='countries', null=False)

    def __str__(self):
        lang = get_language_database()
        lang_obj = getattr(self, '{}'.format(lang), None)
        if lang_obj and lang_obj.name:
            txt = lang_obj.name
        else:
            txt = self.code
        return u"{}".format(smart_text(txt))

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        return [
            ('code', _('Code'), 100),
            ('continent', _('Continent'), 100),
        ]

    def __searchQ__(self, info, text):
        return {
            'code': models.Q(code__icontains=text),
        }

    def __searchF__(self, info):
        filters = {}
        filters['code'] = (_('Code'), lambda x: Q(code__icontains=x), 'input')
        return filters


class TimeZone(CodenerixModel):
    name = models.CharField(_('Name'), max_length=50, unique=True, blank=False)

    def __str__(self):
        return u"{}".format(smart_text(self.name))

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        return [
            ('name', _('Name'), 100),
        ]

    def __searchQ__(self, info, text):
        return {
            'name': models.Q(name__icontains=text),
        }

    def __searchF__(self, info):
        filters = {}
        filters['name'] = (_('Name'), lambda x: Q(name__icontains=x), 'input')
        return filters


class Region(CodenerixModel):
    country = models.ForeignKey(Country, on_delete=models.CASCADE, verbose_name=_('Country'), null=False, related_name='regions')
    code = models.CharField(_('Code'), max_length=3, blank=False)

    def __str__(self):
        lang = get_language_database()
        lang_obj = getattr(self, '{}'.format(lang), None)
        if lang_obj and lang_obj.name:
            txt = lang_obj.name
        else:
            txt = self.code
        return u"{}".format(smart_text(txt))

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        return [
            ('country', _('Country'), 100),
            ('code', _('Code'), 100),
        ]

    def __searchQ__(self, info, text):
        return {
            'code': models.Q(code__icontains=text),
        }

    def __searchF__(self, info):
        filters = {}
        filters['code'] = (_('Code'), lambda x: Q(code__icontains=x), 'input')
        return filters


class Province(CodenerixModel):
    region = models.ForeignKey(Region, on_delete=models.CASCADE, verbose_name=_('Region'), null=False, related_name='provinces')
    code = models.CharField(_('Code'), max_length=3, blank=False)

    def __str__(self):
        lang = get_language_database()
        lang_obj = getattr(self, '{}'.format(lang), None)
        if lang_obj and lang_obj.name:
            txt = lang_obj.name
        else:
            txt = self.code
        return u"{}".format(smart_text(txt))

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        return [
            ('region', _('Region'), 100),
            ('code', _('Code'), 100),
        ]

    def __searchQ__(self, info, text):
        return {
            'code': models.Q(code__icontains=text),
        }

    def __searchF__(self, info):
        filters = {}
        filters['code'] = (_('Code'), lambda x: Q(code__icontains=x), 'input')
        return filters


class City(CodenerixModel):
    country = models.ForeignKey(Country, on_delete=models.CASCADE, verbose_name=_('Country'), null=False, related_name='cities')
    region = models.ForeignKey(Region, on_delete=models.CASCADE, verbose_name=_('Region'), null=True, related_name='cities')
    province = models.ForeignKey(Province, on_delete=models.CASCADE, verbose_name=_('Province'), null=True, related_name='cities')
    time_zone = models.ForeignKey(TimeZone, on_delete=models.CASCADE, verbose_name=_('Timezone'), null=False, related_name='cities')

    def __str__(self):
        lang = get_language_database()
        lang_obj = getattr(self, '{}'.format(lang), None)
        if lang_obj and lang_obj.name:
            txt = lang_obj.name
        else:
            txt = self.pk
        return u"{}".format(smart_text(txt))

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        return [
            ('country', _('Country'), 100),
            ('time_zone', _('Time zone'), 100),
        ]


class GeoAddress(GenInterface):  # META: Abstract class
    class Meta(GenInterface.Meta):
        abstract = True

    country = models.ForeignKey(Country, related_name='%(app_label)s_%(class)s_geo_addresses', verbose_name=_("Country"), blank=True, null=True, on_delete=models.CASCADE)
    region = models.ForeignKey(Region, related_name='%(app_label)s_%(class)s_geo_addresses', verbose_name=_("Region"), blank=True, null=True, on_delete=models.CASCADE)
    province = models.ForeignKey(Province, related_name='%(app_label)s_%(class)s_geo_addresses', verbose_name=_("Province"), blank=True, null=True, on_delete=models.CASCADE)
    city = models.ForeignKey(City, related_name='%(app_label)s_%(class)s_geo_addresses', verbose_name=_("City"), null=True, on_delete=models.CASCADE)
    town = models.CharField(_("Town"), max_length=250, blank=True, null=True)
    zipcode = models.CharField(_("Zip code"), max_length=6, blank=True, null=True)
    address = models.CharField(_("Address"), max_length=250, blank=True, null=True)
    phone = models.CharField(_("Phone"), max_length=16, blank=True, null=True)

    def __str__(self):
        lang = get_language_database()
        lang_obj = getattr(self, '{}'.format(lang), None)
        if lang_obj and lang_obj.name:
            txt = lang_obj.name
        else:
            txt = self.pk
        return u"{}".format(smart_text(txt))

    def __fields__(self, info):
        return [
            ('country', _("Country")),
            ('region', _("Region")),
            ('province', _("Province")),
            ('town', _("Town")),
            ('city', _('City')),
            ('zipcode', _('Zipcode')),
            ('address', _('Address')),
            ('phone', _('Phone')),
        ]

    def get_address(self):
        return self.address or ''

    def get_zipcode(self):
        return self.zipcode or ''

    def get_city(self):
        return self.city or ''

    def get_province(self):
        return self.province or ''

    def get_country(self):
        return self.country or ''


MODELS = (
    ('continent', 'Continent'),
    ('country', 'Country'),
    ('region', 'Region'),
    ('province', 'Province'),
    ('city', 'City'),
)

for field, model in MODELS:
    for lang_code in settings.LANGUAGES_DATABASES:
        query = "class {}GeoName{}(GenGeoName):\n".format(model, lang_code)
        query += "    {} = models.OneToOneField({}, on_delete=models.CASCADE, blank=False, null=False, related_name='{}')\n".format(field, model, lang_code.lower())
        exec(query)
